package jcifs.auth;

import java.io.IOException;

import java.net.URL;

import java.security.NoSuchAlgorithmException;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

public abstract class TokenFactory {

    private static final TokenFactory META_FACTORY;

    private static final Map FACTORIES;

    static {
        META_FACTORY = new MetaTokenFactory();
        FACTORIES = new HashMap();
        String service = "META-INF/services/" + TokenFactory.class.getName();
        Set locations = new HashSet();
        ClassLoader loader = TokenFactory.class.getClassLoader();
        if (loader != null) {
            try {
                Enumeration resources = loader.getResources(service);
                while (resources.hasMoreElements()) {
                    locations.add(resources.nextElement());
                }
            } catch (IOException ex) { }
        }
        try {
            Enumeration resources = ClassLoader.getSystemResources(service);
            while (resources.hasMoreElements()) {
                locations.add(resources.nextElement());
            }
        } catch (IOException ex) { }
        Iterator iterator = locations.iterator();
        while (iterator.hasNext()) {
            try {
                Properties properties = new Properties();
                properties.load(((URL) iterator.next()).openStream());
                Enumeration classNames = properties.propertyNames();
                while (classNames.hasMoreElements()) {
                    Class factoryClass =
                            Class.forName((String) classNames.nextElement());
                    TokenFactory factory = (TokenFactory)
                            factoryClass.newInstance();
                    FACTORIES.put(factory.getAlgorithm(), factory);
                }
            } catch (Exception ex) { }
        }
    }

    public static String[] getAlgorithms() {
        return (String[]) FACTORIES.keySet().toArray(new String[0]);
    }

    public static TokenFactory getInstance() {
        return META_FACTORY;
    }

    public static TokenFactory getInstance(String algorithm)
            throws NoSuchAlgorithmException {
        TokenFactory factory = (TokenFactory) FACTORIES.get(algorithm);
        if (factory == null) {
            throw new NoSuchAlgorithmException(
                    "No TokenFactory found for algorithm: " + algorithm);
        }
        return factory;
    }

    public abstract String getAlgorithm();

    public abstract Token createToken(byte[] material)
            throws UnrecognizedTokenException;

    public abstract byte[] createMaterial(Token token)
            throws UnrecognizedTokenException;

    private static class MetaTokenFactory extends TokenFactory {

        public String getAlgorithm() {
            return null;
        }

        public Token createToken(byte[] material)
                throws UnrecognizedTokenException {
            Iterator factories = FACTORIES.values().iterator();
            while (factories.hasNext()) {
                try {
                    return ((TokenFactory) factories.next()).createToken(
                            material);
                } catch (UnrecognizedTokenException ex) { }
            }
            throw new UnrecognizedTokenException(
                    "Unable to find a TokenFactory for the provided material.");
        }

        public byte[] createMaterial(Token token)
                throws UnrecognizedTokenException {
            Iterator factories = FACTORIES.values().iterator();
            while (factories.hasNext()) {
                try {
                    return ((TokenFactory) factories.next()).createMaterial(
                            token);
                } catch (UnrecognizedTokenException ex) { }
            }
            throw new UnrecognizedTokenException(
                    "Unable to find a TokenFactory for the provided token.");
        }

    }

}
