package jcifs.auth.ntlm;

import java.net.UnknownHostException;

import jcifs.netbios.NbtAddress;

import jcifs.Config;

public class Type1Token implements NtlmToken {

    private int flags;

    private String suppliedDomain;

    private String suppliedWorkstation;

    public Type1Token() {
        this(getDefaultFlags(), getDefaultDomain(), getDefaultWorkstation());
    }

    public Type1Token(int flags, String suppliedDomain,
            String suppliedWorkstation) {
        setFlags(flags);
        setSuppliedDomain(suppliedDomain);
        setSuppliedWorkstation(suppliedWorkstation);
    }

    public int getFlags() {
        return flags;
    }

    public void setFlags(int flags) {
        this.flags = flags;
    }

    public boolean getFlag(int flag) {
        return (getFlags() & flag) != 0;
    }

    public void setFlag(int flag, boolean value) {
        setFlags(value ? (getFlags() | flag) :
                (getFlags() & (0xffffffff ^ flag)));
    }

    public String getSuppliedDomain() {
        return suppliedDomain;
    }

    public void setSuppliedDomain(String suppliedDomain) {
        this.suppliedDomain = suppliedDomain;
    }

    public String getSuppliedWorkstation() {
        return suppliedWorkstation;
    }

    public void setSuppliedWorkstation(String suppliedWorkstation) {
        this.suppliedWorkstation = suppliedWorkstation;
    }

    public static int getDefaultFlags() {
        int flags = NTLMSSP_NEGOTIATE_NTLM | NTLMSSP_REQUEST_TARGET |
                NTLMSSP_NEGOTIATE_ALWAYS_SIGN;
        flags |= Config.getBoolean("jcifs.smb.client.useUnicode", true) ?
                NTLMSSP_NEGOTIATE_UNICODE : NTLMSSP_NEGOTIATE_OEM;
        return flags;
    }

    public static String getDefaultDomain() {
        return Config.getProperty("jcifs.smb.client.domain", null);
    }

    public static String getDefaultWorkstation() {
        try {
            return NbtAddress.getLocalHost().getHostName();
        } catch (UnknownHostException ex) {
            return null;
        }
    }

    public String toString() {
        String suppliedDomain = getSuppliedDomain();
        String suppliedWorkstation = getSuppliedWorkstation();
        int flags = getFlags();
        StringBuffer buffer = new StringBuffer();
        if (suppliedDomain != null) {
            buffer.append("suppliedDomain: ").append(suppliedDomain);
        }
        if (suppliedWorkstation != null) {
            if (buffer.length() > 0) buffer.append("; ");
            buffer.append("suppliedWorkstation: ").append(suppliedWorkstation);
        }
        if (flags != 0) {
            if (buffer.length() > 0) buffer.append("; ");
            buffer.append("flags: ");
            buffer.append("0x");
            buffer.append(Integer.toHexString((flags >> 28) & 0x0f));
            buffer.append(Integer.toHexString((flags >> 24) & 0x0f));
            buffer.append(Integer.toHexString((flags >> 20) & 0x0f));
            buffer.append(Integer.toHexString((flags >> 16) & 0x0f));
            buffer.append(Integer.toHexString((flags >> 12) & 0x0f));
            buffer.append(Integer.toHexString((flags >> 8) & 0x0f));
            buffer.append(Integer.toHexString((flags >> 4) & 0x0f));
            buffer.append(Integer.toHexString(flags & 0x0f));
        }
        return buffer.toString();
    }

}
