package jcifs.auth.ntlm;

import jcifs.Config;

public class Type2Token implements NtlmToken {

    private int flags;

    private byte[] challenge;

    private String target;

    public Type2Token() {
        this(getDefaultFlags(), null, getDefaultDomain());
    }

    public Type2Token(Type1Token type1) {
        this(type1, null, getDefaultDomain());
    }

    public Type2Token(Type1Token type1, byte[] challenge, String target) {
        this(getDefaultFlags(type1), challenge, target);
    }

    public Type2Token(int flags, byte[] challenge, String target) {
        setFlags(flags);
        setChallenge(challenge);
        setTarget(target);
    }

    public int getFlags() {
        return flags;
    }

    public void setFlags(int flags) {
        this.flags = flags;
    }

    public boolean getFlag(int flag) {
        return (getFlags() & flag) != 0;
    }

    public void setFlag(int flag, boolean value) {
        setFlags(value ? (getFlags() | flag) :
                (getFlags() & (0xffffffff ^ flag)));
    }

    public byte[] getChallenge() {
        return challenge;
    }

    public void setChallenge(byte[] challenge) {
        this.challenge = challenge;
    }

    public String getTarget() {
        return target;
    }

    public void setTarget(String target) {
        this.target = target;
    }

    public static int getDefaultFlags() {
        int flags = NTLMSSP_NEGOTIATE_NTLM;
        flags |= Config.getBoolean("jcifs.smb.client.useUnicode", true) ?
                NTLMSSP_NEGOTIATE_UNICODE : NTLMSSP_NEGOTIATE_OEM;
        return flags;
    }

    public static int getDefaultFlags(Type1Token type1) {
        if (type1 == null) return getDefaultFlags();
        int flags = NTLMSSP_NEGOTIATE_NTLM;
        int type1Flags = type1.getFlags();
        flags |= ((type1Flags & NTLMSSP_NEGOTIATE_UNICODE) != 0) ?
                NTLMSSP_NEGOTIATE_UNICODE : NTLMSSP_NEGOTIATE_OEM;
        if ((type1Flags & NTLMSSP_REQUEST_TARGET) != 0) {
            flags |= NTLMSSP_REQUEST_TARGET;
            String domain = getDefaultDomain();
            if (domain != null) flags |= NTLMSSP_TARGET_TYPE_DOMAIN;
        }
        return flags;
    }

    public static String getDefaultDomain() {
        return Config.getProperty("jcifs.smb.client.domain", null);
    }

    public String toString() {
        String target = getTarget();
        byte[] challenge = getChallenge();
        int flags = getFlags();
        StringBuffer buffer = new StringBuffer();
        if (target != null) {
            buffer.append("target: ").append(target);
        }
        if (challenge != null) {
            if (buffer.length() > 0) buffer.append("; ");
            buffer.append("challenge: ");
            buffer.append("0x");
            for (int i = 0; i < challenge.length; i++) {
                buffer.append(Integer.toHexString((challenge[i] >> 4) & 0x0f));
                buffer.append(Integer.toHexString(challenge[i] & 0x0f));
            }
        }
        if (flags != 0) {
            if (buffer.length() > 0) buffer.append("; ");
            buffer.append("flags: ");
            buffer.append("0x");
            buffer.append(Integer.toHexString((flags >> 28) & 0x0f));
            buffer.append(Integer.toHexString((flags >> 24) & 0x0f));
            buffer.append(Integer.toHexString((flags >> 20) & 0x0f));
            buffer.append(Integer.toHexString((flags >> 16) & 0x0f));
            buffer.append(Integer.toHexString((flags >> 12) & 0x0f));
            buffer.append(Integer.toHexString((flags >> 8) & 0x0f));
            buffer.append(Integer.toHexString((flags >> 4) & 0x0f));
            buffer.append(Integer.toHexString(flags & 0x0f));
        }
        return buffer.toString();
    }

}
